@@ -1,7 +1,7 @@
 from functools import wraps
 import httpx
 from pydantic import parse_obj_as
-from .types import Beatmap, Beatmaps, Rankings, User, Scores, Score, GameMode, GameModeInt, RankingType, ScoreTypes, BeatmapUserScore, BeatmapScores, Attributes, KudosuHistory, BeatmapType, BeatmapPlaycount, Beatmapset, UserCompact, Users, Event, Spotlights
+from .types import Beatmap, Beatmaps, Rankings, User, Scores, Score, GameMode, GameModeInt, RankingType, ScoreTypes, BeatmapUserScore, BeatmapScores, Attributes, KudosuHistory, BeatmapType, BeatmapPlaycount, Beatmapset, Users, Event, Spotlights
 from .utility import c_TypeError
 
 class OsuApi:
@@ -19,8 +19,15 @@ class OsuApi:
         self.client_secret = client_secret
         self.authorization = self.__new_auth()
 
-    #? https://osu.ppy.sh/docs/index.html#client-credentials-grant
     def __new_auth(self):
+        """ 
+        returns string in the format "Bearer {{ token }}".
+        
+        Uses the client_id and client_secret set by user
+        to retrieve an auth token for the Osu! api.
+        
+        Api documentation: https://osu.ppy.sh/docs/index.html#client-credentials-grant
+        """
         body_params = {
             "client_id" : self.client_id,
             "client_secret" : self.client_secret,
@@ -34,6 +41,13 @@ class OsuApi:
         return response["token_type"] +" "+ response["access_token"]
 
     def __del_auth(self):
+        """
+        Sets self.authorization to None.
+        
+        Revokes authentication for current token.
+        
+        Api documentation: https://osu.ppy.sh/docs/index.html#revoke-current-token
+        """
         if self.authorization == None:
             raise ValueError("self.authorization is type<None> or is not set.")
 
@@ -44,14 +58,16 @@ class OsuApi:
         self.authorization = None
 
     def __del__(self):
-        self.__del_auth()
-        self.Client.close()
-    
-    def __exit__(self):
+        """
+        Runs self.__del_auth and httpx.Client.Close on garbage collection.
+        """
         self.__del_auth()
         self.Client.close()
 
     def verify_auth(func):
+        """
+        Verifies that Auth token exists and adds it to a headers dictionary
+        """
         @wraps(func)
         def wrapper(self, *args, **kwargs):
             if not self.authorization:
@@ -68,6 +84,18 @@ class OsuApi:
                        checksum:str=None, 
                        filename:str=None,
                        **kwargs) -> Beatmap:
+        """
+        Returns losuapi.types.Beatmap object.
+        
+        Api documentation: https://osu.ppy.sh/docs/index.html#lookup-beatmap
+        
+        parameters:
+            beatmap_id: int - id of Osu! beatmap
+            checksum: str - Osu! beatmap checksum value
+            filename: str - filename to lookup
+        returns:
+            losuapi.types.Beatmap
+        """
         query_params = {}
 
         if not isinstance(beatmap_id, int):
@@ -86,6 +114,8 @@ class OsuApi:
 
 
         response = self.Client.get(url=self.BASE_URL+"/beatmaps/lookup", params=query_params, headers=kwargs["headers"])
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Beatmap, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-a-user-beatmap-score
@@ -116,6 +146,8 @@ class OsuApi:
             query_params["mods"] = mods
 
         response = self.Client.get(url=self.BASE_URL+f"/beatmaps/{beatmap_id}/scores/users/{user_id}", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=BeatmapUserScore, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-a-user-beatmap-scores
@@ -140,6 +172,8 @@ class OsuApi:
             query_params["mode"] = mode
         
         response = self.Client.get(url=self.BASE_URL+f"/beatmaps/{beatmap_id}/scores/users/{user_id}/all", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Scores, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-beatmap-scores
@@ -172,6 +206,8 @@ class OsuApi:
                 raise c_TypeError(param_name="Type",correct="str",wrong=type(Type).__name__)
             
         response = self.Client.get(url=self.BASE_URL+f"/beatmaps/{beatmap_id}/scores", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=BeatmapScores, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-beatmaps
@@ -188,6 +224,8 @@ class OsuApi:
         query_params["ids[]"] = beatmap_ids
 
         response = self.Client.get(url=self.BASE_URL+"/beatmaps", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Beatmaps, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-beatmap
@@ -200,6 +238,8 @@ class OsuApi:
             raise c_TypeError(param_name="beatmap_id", correct="int", wrong=type(beatmap_id).__name__)
         
         response = self.Client.get(url=self.BASE_URL+f"/beatmaps/{beatmap_id}", headers=kwargs["headers"])
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Beatmap, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-beatmap-attributes
@@ -235,6 +275,8 @@ class OsuApi:
             query_params["ruleset_id"] = ruleset_id
 
         response = self.Client.post(url=self.BASE_URL+f"/beatmaps/{beatmap_id}/attributes", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Attributes, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-user-kudosu
@@ -260,6 +302,8 @@ class OsuApi:
             query_params["offset"] = offset
             
         response = self.Client.get(url=self.BASE_URL+f"/users/{user_id}/kudosu", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=list[KudosuHistory], obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-user-scores
@@ -299,6 +343,8 @@ class OsuApi:
             query_params["offset"] = offset
 
         response = self.Client.get(url=self.BASE_URL+f"/users/{user_id}/scores/{Type}", headers=self.base_headers, params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=list[Score], obj=response.json())
                 
     #? https://osu.ppy.sh/docs/index.html#get-user-beatmaps
@@ -329,6 +375,8 @@ class OsuApi:
             query_params["offset"] = offset
             
         response = self.Client.get(url=self.BASE_URL+f"/users/{user_id}/beatmapsets/{Type}", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         if Type == BeatmapType.MOST_PLAYED.value:
             return parse_obj_as(type_=list[BeatmapPlaycount], obj=response.json())
         return parse_obj_as(type_=list[Beatmapset], obj=response.json())
@@ -356,6 +404,8 @@ class OsuApi:
             query_params["offset"] = offset
 
         response = self.Client.get(url=self.BASE_URL+f"/users/{user_id}/recent_activity", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=list[Event], obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-user
@@ -382,6 +432,8 @@ class OsuApi:
             query_params["key"] = key
 
         response = self.Client.get(url=self.BASE_URL+f"/users/{username}/{mode}", headers=kwargs["headers"],params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=User, obj=response.json())
 
     #? https://osu.ppy.sh/docs/index.html#get-users
@@ -398,6 +450,8 @@ class OsuApi:
         query_params["ids[]"] = user_ids
 
         response = self.Client.get(url=self.BASE_URL+"/users", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Users, obj=response.json())
 
     #? https://osu.ppy.sh/docs/index.html#get-ranking
@@ -456,6 +510,8 @@ class OsuApi:
             query_params["variant"] = variant
 
         response = self.Client.get(url=self.BASE_URL+f"/rankings/{mode}/{Type}", headers=kwargs["headers"], params=query_params)
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Rankings, obj=response.json())
     
     #? https://osu.ppy.sh/docs/index.html#get-spotlights
@@ -463,4 +519,6 @@ class OsuApi:
     def spotlights(self, **kwargs)->Spotlights:
 
         response = self.Client.get(url=self.BASE_URL+"/spotlights", headers=kwargs["headers"])
+        if "error" in response.json():
+            return None
         return parse_obj_as(type_=Spotlights, obj=response.json())
\ No newline at end of file
